use std::fmt::Display;

use libpt::log::{error, info};
use thiserror::Error;
use warp::http::StatusCode;
use warp::reject;

#[derive(Debug, Error)]
pub struct TemplateError(#[from] minijinja::Error);

impl reject::Reject for TemplateError {}
impl Display for TemplateError {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "something went wrong with producing this page")
    }
}

pub async fn handle_rejection(
    err: reject::Rejection,
) -> Result<impl warp::reply::Reply, std::convert::Infallible> {
    let code;
    let message;
    info!("rejecting: {err:?}");

    if err.is_not_found() {
        code = StatusCode::NOT_FOUND;
        message = "page not found";
    } else if let Some(e) = err.find::<TemplateError>() {
        error!("templating error: {e}");
        code = StatusCode::INTERNAL_SERVER_ERROR;
        message = "could not process data to make a page";
    } else {
        error!("unhandled rejection: {:?}", err);
        code = StatusCode::INTERNAL_SERVER_ERROR;
        message = "unhandled rejection";
    }

    Ok(warp::reply::with_status(message, code))
}
